import { Directive, forwardRef, Renderer2, ElementRef, Input, HostListener, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR, NG_ASYNC_VALIDATORS, FormsModule, ReactiveFormsModule } from '@angular/forms';
import { of, forkJoin, fromEventPattern, ReplaySubject } from 'rxjs';
import { take, map, shareReplay, first } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FileInputAccessor {
    /**
     * @param {?} _renderer
     * @param {?} _elementRef
     */
    constructor(_renderer, _elementRef) {
        this._renderer = _renderer;
        this._elementRef = _elementRef;
        this.onChange = (/**
         * @param {?} _
         * @return {?}
         */
        (_) => { });
        this.onTouched = (/**
         * @return {?}
         */
        () => { });
        this.validator = this.generateAsyncValidator();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set allowedExt(value) {
        if (typeof value === 'string') {
            value = value + '$';
        }
        if (value instanceof Array) {
            value = value.join('|') + '$';
        }
        this._allowedExt = value;
    }
    /**
     * @return {?}
     */
    get allowedExt() {
        return this._allowedExt;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this._renderer.setProperty(this._elementRef.nativeElement, 'value', null);
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = this.onChangeGenerator(fn);
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) { }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        this._renderer.setProperty(this._elementRef.nativeElement, 'disabled', isDisabled);
    }
    /**
     * @param {?} c
     * @return {?}
     */
    validate(c) {
        return this.validator(c);
    }
    /**
     * Generator method that I used to move the code for the AsyncValidator down here so it didn't
     * get in my way, way up there ^.
     * @private
     * @return {?}
     */
    generateAsyncValidator() {
        return (/**
         * @param {?} c
         * @return {?}
         */
        (c) => {
            if (!c.value || !c.value.length || c.disabled)
                return of({});
            /** @type {?} */
            const errors = {};
            /** @type {?} */
            const loaders = [];
            for (const f of c.value) {
                if (this.size && this.size < f.size) {
                    f.errors.fileSize = true;
                    errors.fileSize = true;
                }
                if (f.isImg && (this.maxWidth || this.maxHeight)) {
                    loaders.push(f.imgLoadReplay
                        .pipe(take(1), map((/**
                     * @param {?} e
                     * @return {?}
                     */
                    (e) => {
                        /** @type {?} */
                        const minWidthError = this.minWidth && f.imgWidth < this.minWidth;
                        /** @type {?} */
                        const minHeightError = this.minHeight && f.imgHeight < this.minHeight;
                        /** @type {?} */
                        const maxWidthError = this.maxWidth && f.imgWidth > this.maxWidth;
                        /** @type {?} */
                        const maxHeightError = this.maxHeight && f.imgHeight > this.maxHeight;
                        if (minWidthError) {
                            f.errors.minWidth = true;
                            errors.minWidth = true;
                        }
                        if (minHeightError) {
                            f.errors.minHeight = true;
                            errors.minHeight = true;
                        }
                        if (maxWidthError) {
                            f.errors.maxWidth = true;
                            errors.maxWidth = true;
                        }
                        if (maxHeightError) {
                            f.errors.maxHeight = true;
                            errors.maxHeight = true;
                        }
                        /** will be @deprecated **/
                        if (minWidthError || maxWidthError) {
                            f.errors.imageWidth = true;
                            errors.imageWidth = true;
                        }
                        /** will be @deprecated **/
                        if (minHeightError || maxHeightError) {
                            f.errors.imageHeight = true;
                            errors.imageHeight = true;
                        }
                        return e;
                    }))));
                }
                if (!this.allowedExt && !this.allowedTypes)
                    continue;
                /** @type {?} */
                const extP = this.generateRegExp(this.allowedExt);
                /** @type {?} */
                const typeP = this.generateRegExp(this.allowedTypes);
                if (extP && !extP.test(f.name)) {
                    f.errors.fileExt = true;
                    errors.fileExt = true;
                }
                if (typeP && !typeP.test(f.type)) {
                    f.errors.fileType = true;
                    errors.fileType = true;
                }
            }
            if (loaders.length) {
                return forkJoin(...loaders).pipe(map((/**
                 * @return {?}
                 */
                () => errors)));
            }
            return of(errors);
        });
    }
    /**
     * Generator method that returns an onChange handler
     * @private
     * @param {?} fn
     * @return {?}
     */
    onChangeGenerator(fn) {
        return (/**
         * @param {?} files
         * @return {?}
         */
        (files) => {
            /** @type {?} */
            const fileArr = [];
            for (const f of files) {
                if (this.withMeta && FileReader) {
                    /** @type {?} */
                    const fr = new FileReader();
                    this.generateFileMeta(f, fr);
                }
                f.errors = {};
                fileArr.push(f);
            }
            fn(fileArr);
        });
    }
    /**
     * @private
     * @param {?} pattern
     * @return {?}
     */
    generateRegExp(pattern) {
        if (!pattern)
            return null;
        if (pattern instanceof RegExp) {
            return new RegExp(pattern);
        }
        else if (typeof pattern === 'string') {
            return new RegExp(pattern, 'ig');
        }
        else if (pattern instanceof Array) {
            return new RegExp(`(${pattern.join('|')})`, 'ig');
        }
        return null;
    }
    /**
     * The ICustomFile has a ReplaySubject property for text / image files that will emit
     * once the file has been loaded. Might get removed later since I haven't found a use for it yet.
     * @private
     * @param {?} f
     * @param {?} fr
     * @return {?}
     */
    generateFileMeta(f, fr) {
        if (f.type.match(/text.*/)) {
            f.textLoadReplay = this.setText(f, fr);
        }
        else if (f.type.match(/image.*/)) {
            f.imgLoadReplay = this.setImage(f, fr);
        }
    }
    /**
     * @private
     * @param {?} f
     * @param {?} fr
     * @return {?}
     */
    setImage(f, fr) {
        f.isImg = true;
        /** @type {?} */
        const img = new Image();
        /** @type {?} */
        const imgLoadObs = fromEventPattern(((/**
         * @param {?} handler
         * @return {?}
         */
        (handler) => img.addEventListener('load', handler))), ((/**
         * @param {?} handler
         * @return {?}
         */
        (handler) => img.removeEventListener('load', handler)))).pipe(take(1), shareReplay());
        /** @type {?} */
        const frLoadObs = fromEventPattern(((/**
         * @param {?} handler
         * @return {?}
         */
        (handler) => fr.addEventListener('load', handler))), ((/**
         * @param {?} handler
         * @return {?}
         */
        (handler) => fr.removeEventListener('load', handler)))).pipe(take(1), shareReplay());
        /** @type {?} */
        const onloadReplay = new ReplaySubject(1);
        forkJoin(imgLoadObs, frLoadObs).pipe(first()).subscribe(onloadReplay);
        imgLoadObs.pipe(first()).subscribe((/**
         * @return {?}
         */
        () => {
            f.imgHeight = img.height;
            f.imgWidth = img.width;
        }));
        frLoadObs.pipe(first()).subscribe((/**
         * @return {?}
         */
        () => {
            f.imgSrc = fr.result + '';
            img.src = fr.result + '';
        }));
        fr.readAsDataURL(f);
        return onloadReplay;
    }
    /**
     * @private
     * @param {?} f
     * @param {?} fr
     * @return {?}
     */
    setText(f, fr) {
        /** @type {?} */
        const frLoadObs = fromEventPattern(((/**
         * @param {?} handler
         * @return {?}
         */
        (handler) => fr.addEventListener('load', handler))), ((/**
         * @param {?} handler
         * @return {?}
         */
        (handler) => fr.removeEventListener('load', handler)))).pipe(take(1), shareReplay());
        /** @type {?} */
        const onloadReplay = new ReplaySubject(1);
        frLoadObs.subscribe(onloadReplay);
        frLoadObs.pipe(first()).subscribe((/**
         * @return {?}
         */
        () => {
            f.textContent = fr.result + '';
        }));
        fr.readAsText(f);
        return onloadReplay;
    }
}
FileInputAccessor.decorators = [
    { type: Directive, args: [{
                selector: 'input[type=file][formControl],input[type=file][formControlName],input[type=file][ngModel]',
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef((/**
                         * @return {?}
                         */
                        () => FileInputAccessor)),
                        multi: true
                    },
                    {
                        provide: NG_ASYNC_VALIDATORS,
                        useExisting: forwardRef((/**
                         * @return {?}
                         */
                        () => FileInputAccessor)),
                        multi: true
                    }
                ]
            },] }
];
/** @nocollapse */
FileInputAccessor.ctorParameters = () => [
    { type: Renderer2 },
    { type: ElementRef }
];
FileInputAccessor.propDecorators = {
    allowedTypes: [{ type: Input }],
    size: [{ type: Input }],
    withMeta: [{ type: Input }],
    maxHeight: [{ type: Input }],
    maxWidth: [{ type: Input }],
    minHeight: [{ type: Input }],
    minWidth: [{ type: Input }],
    allowedExt: [{ type: Input }],
    onChange: [{ type: HostListener, args: ['change', ['$event.target.files'],] }],
    onTouched: [{ type: HostListener, args: ['blur',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FileInputAccessorModule {
}
FileInputAccessorModule.decorators = [
    { type: NgModule, args: [{
                declarations: [FileInputAccessor],
                imports: [
                    FormsModule,
                    ReactiveFormsModule
                ],
                exports: [
                    FileInputAccessor,
                    FormsModule,
                    ReactiveFormsModule
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { FileInputAccessor, FileInputAccessorModule };
//# sourceMappingURL=file-input-accessor.js.map
